/*****************************************************************************/
/* admin.c : Administration du serveur
 * admin.c : Server administration
 *
 *
 * hOpla : Lien XML-SQL - XML-SQL link
 * Copyright (c) 2000 Philippe Roy
 * Auteur - Author : Philippe Roy
 *
 *
 * Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier
 * sous les termes de la licence publique gnrale GNU telle qu'elle est publie par
 * la Free Software Foundation ; soit la version 2 de la licence, ou
 * (comme vous voulez) toute version ultrieure.
 *
 * Ce programme est distribu dans l'espoir qu'il sera utile,
 * mais SANS AUCUNE GARANTIE ; mme sans la garantie de
 * COMMERCIALIT ou d'ADQUATION A UN BUT PARTICULIER. Voir la
 * licence publique gnrale GNU pour plus de dtails.
 *
 * Vous devriez avoir reu une copie de la licence publique gnrale GNU
 * avec ce programme ; si ce n'est pas le cas, crivez  la Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/*****************************************************************************/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "main.h"


/*****************************************************************************/
/* Test de conformit du fichier */
/*****************************************************************************/
gboolean check_file (Td_xmlbase base)
{
  Td_xmltable table = NULL;
  Td_xmlfield field = NULL;
  Td_xmldatatable datatable = NULL;
  gboolean erreur;
  table = g_malloc0 (sizeof (Td_xmltable_tmp));
  field = g_malloc0 (sizeof (Td_xmlfield_tmp));
  datatable = g_malloc0 (sizeof (Td_xmldatatable_tmp));
  erreur = FALSE;

  /* Base */
  if (strcmp (base->name, "database administration"))
    erreur = TRUE;
  if (strcmp (base->comment, "(null)"))
    erreur = TRUE;
  if (base->nbtables != 1)
    erreur = TRUE;
  if (base->nbviews != 0)
    erreur = TRUE;
  if (base->nbdatatables != 1)
    erreur = TRUE;

  /* Table */
  table = base->tables[0];
  if (strcmp (table->name, "hopla_bases"))
    erreur = TRUE;
  if (strcmp (table->comment, "(null)"))
    erreur = TRUE;
  if (table->nbfields != 1)
    erreur = TRUE;

  /* Champs */
  field = table->fields[0];
  if (strcmp (field->name, "name"))
    erreur = TRUE;
  if (strcmp (field->comment, "(null)"))
    erreur = TRUE;
  if (field->key != 1)
    erreur = TRUE;
  if (strcmp (field->type, "varchar[200]"))
    erreur = TRUE;
  if (strcmp (field->type_comment, "(null)"))
    erreur = TRUE;

  /* Table de donnes */
  datatable = base->datatables[0];
  if (strcmp (datatable->table_source, "hopla_bases"))
    erreur = TRUE;

  /* Fin */
  if (erreur)
    {
      g_free (field);
      g_free (table);
      return FALSE;
    }
  g_free (field);
  g_free (table);
  return TRUE;
}


/*****************************************************************************/
/* Ecriture */
/*****************************************************************************/
gboolean xml2admin_main (void)
{
  Td_xmlbase new_base;
  Td_xmldatatable old_datatable;
  Td_xmldatatable new_datatable;
  gchar *txt_tampon;

  /* Lecture du fichier */
  txt_tampon = g_strdup_printf (_("Reading file '%s'"), HOPLA_FILE_POPT);
  hopla_message (txt_tampon, "begin");
  new_base = td_parse_xmlbase (HOPLA_FILE_POPT);
  if (!new_base)
    {
      hopla_message (txt_tampon, "failed");
      hopla_message (txt_tampon, "end");
      return FALSE;
    }
  hopla_message (txt_tampon, "done");
  hopla_message (txt_tampon, "end");
  new_datatable = new_base->datatables[0];

  /* Vrification du fichier par rapport  la base de donnes */
  txt_tampon = g_strdup_printf (_("Checking file"));
  hopla_message (txt_tampon, "begin");
  if (!check_file (new_base))
    {
      hopla_message (txt_tampon, "failed");
      hopla_message (txt_tampon, "end");
      return FALSE;
    }
  hopla_message (txt_tampon, "done");
  hopla_message (txt_tampon, "end");

  /* Lecture des donnes administrateur du gestionnaire de base de donnes */
  txt_tampon = g_strdup_printf (_("Reading database's administration info"));
  hopla_message (txt_tampon, "begin");
  old_datatable = hopla_admin_read ();
  if (!old_datatable)
    {
      hopla_message (txt_tampon, "failed");
      hopla_message (txt_tampon, "end");
      return FALSE;
    }
  hopla_message (txt_tampon, "done");
  hopla_message (txt_tampon, "end");

  /* Ecriture des donnes administrateur du gestionnaire de base de donnes */
  txt_tampon = g_strdup_printf (_("Writting database's administration info"));
  hopla_message (txt_tampon, "begin");
  if (!hopla_admin_write (new_datatable, old_datatable))
    {
      hopla_message (txt_tampon, "failed");
      hopla_message (txt_tampon, "end");
      return FALSE;
    }
  hopla_message (txt_tampon, "done");
  hopla_message (txt_tampon, "end");
  return TRUE;
}

/*****************************************************************************/
/* Lecture */
/*****************************************************************************/
gboolean admin2xml_main (void)
{
  Td_xmlbase base = NULL;
  Td_xmltable table = NULL;
  Td_xmlfield field = NULL;
  Td_xmldatatable datatable = NULL;
  gchar *txt_tampon;
  base = g_malloc0 (sizeof (Td_xmlbase_tmp));
  table = g_malloc0 (sizeof (Td_xmltable_tmp));
  field = g_malloc0 (sizeof (Td_xmlfield_tmp));

  /* Definition de la structure XML */
  base->name = g_strdup_printf ("database administration");
  base->comment = g_strdup_printf ("(null)");
  table->name = g_strdup_printf ("hopla_bases");
  table->comment = g_strdup_printf ("(null)");
  field->name = g_strdup_printf ("name");
  field->comment = g_strdup_printf ("(null)");
  field->key = 1;
  field->type = g_strdup_printf ("varchar[200]");
  field->type_comment = g_strdup_printf ("(null)");
  table->fields[0] = field;
  table->nbfields = 1;
  base->tables[0] = table;
  base->nbtables = 1;
  
  /* Lecture des donnes administrateur du gestionnaire de base de donnes */
  /* FIXME: pour le momment limite aux tables, le reste (utilisateur, groupe, permissions) +tards */
  txt_tampon = g_strdup_printf (_("Reading database's administration info"));
  hopla_message (txt_tampon, "begin");
  base->datatables[0] = hopla_admin_read ();
  base->datatables[0]->table_source = g_strdup_printf ("hopla_bases");
  base->nbdatatables = 1;
  if (!base->datatables[0])
    {
      hopla_message (txt_tampon, "failed");
      hopla_message (txt_tampon, "end");
      g_free (field);
      g_free (table);
      g_free (base);
      return FALSE;
    }
  hopla_message (txt_tampon, "done");
  hopla_message (txt_tampon, "end");

  /* Exportation vers le fichier XML */
  txt_tampon = g_strdup_printf (_("Writing file '%s'"), HOPLA_FILE_POPT);
  hopla_message (txt_tampon, "begin");
  if (!td_save_xmlbase (HOPLA_FILE_POPT, base, "base"))
    {
      hopla_message (txt_tampon, "failed");
      hopla_message (txt_tampon, "end");
      g_free (field);
      g_free (table);
      g_free (base);
      return FALSE;
    }
  hopla_message (txt_tampon, "done");
  hopla_message (txt_tampon, "end");
  g_free (field);
  g_free (table);
  g_free (base);
  return TRUE;
}





