/*****************************************************************************/
/* db2xml.c : Exporte les donnes de la base de donnes d'une base vers un fichier
 * db2xml.c : Export a database's data from base to file
 *
 *
 * hOpla : Lien XML-SQL - XML-SQL link
 * Copyright (c) 2000 Philippe Roy
 * Auteur - Author : Philippe Roy
 *
 *
 * Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier
 * sous les termes de la licence publique gnrale GNU telle qu'elle est publie par
 * la Free Software Foundation ; soit la version 2 de la licence, ou
 * (comme vous voulez) toute version ultrieure.
 *
 * Ce programme est distribu dans l'espoir qu'il sera utile,
 * mais SANS AUCUNE GARANTIE ; mme sans la garantie de
 * COMMERCIALIT ou d'ADQUATION A UN BUT PARTICULIER. Voir la
 * licence publique gnrale GNU pour plus de dtails.
 *
 * Vous devriez avoir reu une copie de la licence publique gnrale GNU
 * avec ce programme ; si ce n'est pas le cas, crivez  la Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/*****************************************************************************/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include "main.h"


/*****************************************************************************/
/* Structure  */
/*****************************************************************************/
gboolean struc2xml_main (void)
{
  Td_xmlbase base;
  gchar *txt_tampon;

  /* Lecture structure base de donnes */
  txt_tampon = g_strdup_printf (_("Reading database's structure '%s'"), HOPLA_BASE_POPT);
  hopla_message (txt_tampon, "begin");
  base = hopla_parse_dbbase ();
  if (!base)
    {
      hopla_message (txt_tampon, "failed");
      hopla_message (txt_tampon, "end");
      return FALSE;
    }
  hopla_message (txt_tampon, "done");
  hopla_message (txt_tampon, "end");

  /* Exportation vers le fichier XML */
  txt_tampon = g_strdup_printf (_("Writing file '%s'"), HOPLA_FILE_POPT);
  hopla_message (txt_tampon, "begin");
  if (!td_save_xmlbase (HOPLA_FILE_POPT, base, "table"))
    {
      hopla_message (txt_tampon, "failed");
      hopla_message (txt_tampon, "end");
      return FALSE;
    }
  hopla_message (txt_tampon, "done");
  hopla_message (txt_tampon, "end");
  return TRUE;
}


/*****************************************************************************/
/* Donnes */
/*****************************************************************************/
gboolean data2xml_main (void)
{
  Td_xmlbase base = NULL;
  Td_xmldatatable datatable = NULL;
  char *requete;
  gchar *txt_tampon;
  base = g_malloc0 (sizeof (Td_xmlbase_tmp));

  /* Execution de la requte SQL */
  requete = g_strdup_printf ("%s", HOPLA_QUERY_POPT);
  txt_tampon = g_strdup_printf (_("Executing query '%s'"), requete);
  hopla_message (txt_tampon, "begin");
  base->name = g_strdup_printf ("%s", HOPLA_BASE_POPT);
  base->comment = "(bud)";
  base->datatables[0] = hopla_query (requete);
  base->nbdatatables = 1;
  if (!base->datatables[0])
    {
      hopla_message (txt_tampon, "failed");
      hopla_message (txt_tampon, "end");
      g_free (base);
      return FALSE;
    }
  hopla_message (txt_tampon, "done");
  hopla_message (txt_tampon, "end");

  /* Exportation vers le fichier XML */
  txt_tampon = g_strdup_printf (_("Writing file '%s'"), HOPLA_FILE_POPT);
  hopla_message (txt_tampon, "begin");
  if (!td_save_xmlbase (HOPLA_FILE_POPT, base, "datatable"))
    {
      hopla_message (txt_tampon, "failed");
      hopla_message (txt_tampon, "end");
      g_free (base);
      return FALSE;
    }
  hopla_message (txt_tampon, "done");
  hopla_message (txt_tampon, "end");
  g_free (base);
  return TRUE;
}


/*****************************************************************************/
/* Base */
/*****************************************************************************/
gboolean check_template (Td_xmlbase base, Td_xmlbase template)
{
  Td_xmltable table;
  Td_xmlview view;
  Td_xmlfield field;
  Td_xmlviewfield viewfield;
  int i, j, k;
  gboolean erreur;
  gchar *txt_tampon = NULL;

  /* Vue */
  for (k=0; k < template->nbviews; k++)
    {
      view = template->views[k];
      txt_tampon = g_strdup_printf (_("Checking template : '%s' (view) defined in database ? ..."), view->name);
      erreur = TRUE;
      for (i=0; i < base->nbtables; i++)
	{
	  table = base->tables[i];
	  if (!strcmp (table->name, view->name))
	    erreur = FALSE;
	}
      if (erreur)
	{
	  hopla_message (txt_tampon, "failed");
	  return FALSE;
	}
      hopla_message (txt_tampon, "done");
    }

  /* Champs */
  for (k=0; k < template->nbviews; k++)
    {
      view = template->views[k];
      i =0;
      do
	{
	  table = base->tables[i];
	  i++;
	}
      while (strcmp (table->name, view->name));
      for (i=0; i < view->nbfields; i++)
	{
	  erreur = TRUE;
	  viewfield = view->fields[i];
	  for (j=0; j < table->nbfields; j++)
	    {
	      field = table->fields[j];
	      if (!strcmp (field->name, viewfield->name))
		erreur = FALSE;
	    }
	  txt_tampon = g_strdup_printf (_("Checking template : '%s.%s' (field) defined in database ? ..."), view->name, viewfield->name);
	  if (erreur)
	    {
	      hopla_message (txt_tampon, "failed");
	      return FALSE;
	    }
	  hopla_message (txt_tampon, "done");
	}
    }
  return TRUE;
}

gboolean db2xml_main (void)
{
  Td_xmlbase base = NULL;
  Td_xmlbase template = NULL;
  Td_xmlbase base_ret;
  gchar *txt_tampon;

  /* Avec gabarit */
  if (HOPLA_TEMPLATE_POPT)
    {

      /* Lecture du gabarit */
      txt_tampon = g_strdup_printf (_("Reading template '%s'"), HOPLA_TEMPLATE_POPT);
      hopla_message (txt_tampon, "begin");
      template = td_parse_xmlbase (HOPLA_TEMPLATE_POPT);
      if (!template)
	{
	  hopla_message (txt_tampon, "failed");
	  hopla_message (txt_tampon, "end");
	  return FALSE;
	}
      hopla_message (txt_tampon, "done");
      hopla_message (txt_tampon, "end");

      /* Verification du gabarit */
      txt_tampon = g_strdup_printf (_("Checking template file"));
      hopla_message (txt_tampon, "begin");
      base = td_parse_xmlbase (HOPLA_TEMPLATE_POPT);
      if (!base)
	{
	  hopla_message (txt_tampon, "failed");
	  hopla_message (txt_tampon, "end");
	  return FALSE;
	}
      hopla_message (txt_tampon, "done");
      hopla_message (txt_tampon, "end");
      
      /* Lecture structure base de donnes */
      txt_tampon = g_strdup_printf (_("Reading database's structure '%s'"), HOPLA_BASE_POPT);
      hopla_message (txt_tampon, "begin");
      base = hopla_parse_dbbase ();
      if (!base)
	{
	  hopla_message (txt_tampon, "failed");
	  hopla_message (txt_tampon, "end");
	  return FALSE;
	}
      hopla_message (txt_tampon, "done");
      hopla_message (txt_tampon, "end");

      /* Vrification de la compatibilit entre gabarit et base de donnes */
      txt_tampon = g_strdup_printf (_("Checking template/database compatibility"));
      hopla_message (txt_tampon, "begin");
      if (!check_template (base, template))
	{
	  hopla_message (txt_tampon, "failed");
	  hopla_message (txt_tampon, "end");
	  return FALSE;
	}
      hopla_message (txt_tampon, "done");
      hopla_message (txt_tampon, "end");
    }

  /* Lecture de la base de donnes */
  txt_tampon = g_strdup_printf (_("Reading data base '%s'"), HOPLA_BASE_POPT);
  hopla_message (txt_tampon, "begin");
  if (HOPLA_TEMPLATE_POPT)
    base_ret = hopla_read_dbbase (template);
  else
    base_ret = hopla_read_dbbase (NULL);
  if (!base_ret)
    {
      hopla_message (txt_tampon, "failed");
      hopla_message (txt_tampon, "end");
      return FALSE;
    }
  hopla_message (txt_tampon, "done");
  hopla_message (txt_tampon, "end");

  /* Exportation vers le fichier XML */
  txt_tampon = g_strdup_printf (_("Writing file '%s'"), HOPLA_FILE_POPT);
  hopla_message (txt_tampon, "begin");
  if (!td_save_xmlbase (HOPLA_FILE_POPT, base_ret, "base"))
    {
      hopla_message (txt_tampon, "failed");
      hopla_message (txt_tampon, "end");
      return FALSE;
    }
  hopla_message (txt_tampon, "done");
  hopla_message (txt_tampon, "end");
  return TRUE;
}
