/*****************************************************************************/
/* xml2struc.c : Importation d'une structure de la base de donnes d'un fichier vers une base
 * xml2struc.c : Import a database's structure from file to base
 *
 *
 * hOpla : Lien XML-SQL - XML-SQL link
 * Copyright (c) 2000 Philippe Roy
 * Auteur - Author : Philippe Roy
 *
 *
 * Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier
 * sous les termes de la licence publique gnrale GNU telle qu'elle est publie par
 * la Free Software Foundation ; soit la version 2 de la licence, ou
 * (comme vous voulez) toute version ultrieure.
 *
 * Ce programme est distribu dans l'espoir qu'il sera utile,
 * mais SANS AUCUNE GARANTIE ; mme sans la garantie de
 * COMMERCIALIT ou d'ADQUATION A UN BUT PARTICULIER. Voir la
 * licence publique gnrale GNU pour plus de dtails.
 *
 * Vous devriez avoir reu une copie de la licence publique gnrale GNU
 * avec ce programme ; si ce n'est pas le cas, crivez  la Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/*****************************************************************************/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "main.h"
#include "xml2struc.h"


/*****************************************************************************/
/* Test de conformit du fichier */
/*****************************************************************************/

/* Vrification des noms de tables et champs :
   - pas de doublons sur les noms de table
   - pas de doublons par table sur les noms de champs */

gboolean check_name (Td_xmlbase cur_base)
{
  Td_xmltable verif_table = NULL;
  Td_xmltable cur_table = NULL;
  Td_xmlfield verif_field = NULL;
  Td_xmlfield cur_field = NULL;
  gchar *cur_txt;
  gchar *txt_tampon1;
  gchar *txt_tampon2;
  gboolean erreur;
  int i, j, k;

  /* Pas de doublons sur les noms de table */
  txt_tampon1 = g_strdup_printf (_("Checking table integrity : no tables with same name per base"));
  hopla_message (txt_tampon1, "begin");
  for (i=0; i < cur_base->nbtables; i++)
    {
      verif_table = cur_base->tables[i];
      for (j=0; j < cur_base->nbtables; j++)
	{
	  cur_table = cur_base->tables[j];
	  txt_tampon2 = g_strdup_printf (_("Checking table integrity : '%s':%d<>'%s':%d ?"), verif_table->name, i, cur_table->name, j);
	  if ((!strcmp (verif_table->name, cur_table->name)) && (i != j))
	    {
	      hopla_message (txt_tampon2, "failed");
	      return FALSE;
	    }
	  hopla_message (txt_tampon2, "done");
	}
    }
  hopla_message (txt_tampon1, "end");

  /* Pas de doublons par table sur les noms de champs */
  txt_tampon1 = g_strdup_printf (_("Checking field integrity : no fields with same name per table"));
  hopla_message (txt_tampon1, "begin");
  for (i=0; i < cur_base->nbtables; i++)
    {
      cur_table = cur_base->tables[i];
      for (j=0; j < cur_table->nbfields; j++)
	{
	  verif_field = cur_table->fields[j];
	  for (k=0; k < cur_table->nbfields; k++)
	    {
	      cur_field = cur_table->fields[k];
	      txt_tampon2 = g_strdup_printf (_("Checking field integrity : '%s.%s':%d<>'%s.%s':%d ?"), cur_table->name, verif_field->name, j, cur_table->name, cur_field->name, k);
	      if ((!strcmp (verif_field->name, cur_field->name)) && (j != k))
		{
		  hopla_message (txt_tampon2, "failed");
		  return FALSE;
		}
	      hopla_message (txt_tampon2, "done");
	    }
	}
    }
  hopla_message (txt_tampon1, "end");

  /* Cl existante */
  txt_tampon1 = g_strdup_printf (_("Checking table's key : one key per table"));
  hopla_message (txt_tampon1, "begin");
  for (i=0; i < cur_base->nbtables; i++)
    {
      cur_table = cur_base->tables[i];
      txt_tampon2 = g_strdup_printf (_("Checking table's key : '%s' existing ?"), cur_table->name);
      erreur = TRUE;
      for (j=0; j < cur_table->nbfields; j++)
	{
	  cur_field = cur_table->fields[j];
	  if (cur_field->key)
	    erreur = FALSE;
	}
      if (erreur)
	{
	  hopla_message (txt_tampon2, "failed");
	  return FALSE;
	}
      hopla_message (txt_tampon2, "done");
    }
  
  /* Cl unique par table */
  for (i=0; i < cur_base->nbtables; i++)
    {
      cur_table = cur_base->tables[i];
      erreur = FALSE;
      for (j=0; j < cur_table->nbfields; j++)
	{
	  verif_field = cur_table->fields[j];
	  if (verif_field->key)
	    {
	      txt_tampon2 = g_strdup_printf (_("Checking table's key : '%s.%s' key alone ?"), cur_table->name, verif_field->name);
	      erreur = FALSE;
	      for (k=0; k < cur_table->nbfields; k++)
		{
		  cur_field = cur_table->fields[k];
		  if ((cur_field->key)&&(k != j))
		    erreur = TRUE;
		}
	      if (erreur)
		{
		  hopla_message (txt_tampon2, "failed");
		  return FALSE;
		}
	      hopla_message (txt_tampon2, "done");
	    }
	}
    }
  hopla_message (txt_tampon1, "end");
  return TRUE;
}


/* Vrification des types de champs :
   - type PostgreSQL */

gboolean check_type (Td_xmlbase cur_base)
{
  Td_xmltable cur_table = NULL;
  Td_xmlfield cur_field = NULL;
  gchar *txt_tampon1;
  gchar *txt_tampon2;
  gchar *txt_tampon;
  int i, j, k;
  gboolean erreur;

  /* Type PostgreSQL */
  /* 41 types fixes */
  gchar *pgsqltype_fixe []={"abstime","aclitem","bool","box","bytea","cid","cidr","circle","date","datetime","filename","float4",
			    "float8","inet","int2","int28","int4","int8","line","lseg","macaddr","money","name","numeric","oid","oid8","path","point",
			    "polygon","regproc","reltime","smgr","text","tid","time","timespan","timestamp","tinterval","unknown","varchar","xid"};

  /* 2 types variables */
  gchar *pgsqltype_variable []={"varchar()","char()"};  /* En ralit, "varchar(n)" =" bpchar" et char(n)="char" */
  int N = 1001; /* O n maxi = 1000 : on peut voir venir ! */

  /* Go */
  txt_tampon1 = g_strdup_printf (_("Checking type : all types defined (PostgreSQL)"));
  hopla_message (txt_tampon1, "begin");
  for (i=0; i < cur_base->nbtables; i++)
    {
      cur_table = cur_base->tables[i];
      for (j=0; j < cur_table->nbfields; j++)
	{
	  cur_field = cur_table->fields[j];
	  txt_tampon2 = g_strdup_printf (_("Checking type : '%s.%s.%s' defined ?"), cur_table->name, cur_field->name, cur_field->type);
	  erreur = TRUE;

	  /* Vrif type fixe */
	  for (k=0; k < 41; k++)
	    if ((!strcmp (pgsqltype_variable[k], cur_field->type)))
	      erreur = FALSE;

	  /* Vrif type variable */
	  if (erreur)
	    {
	      for (k=1; k < N; k++)
		{
		  txt_tampon = g_strdup_printf ("char(%d)", k);
		  if (!strcmp (txt_tampon, cur_field->type))
		    erreur = FALSE;
		  txt_tampon = g_strdup_printf ("varchar(%d)", k);
		  if (!strcmp (txt_tampon, cur_field->type))
		    erreur = FALSE;
		}
	    }
	  if (erreur)
	    {
	      hopla_message (txt_tampon2, "failed");
	      return FALSE;
	    }
	  hopla_message (txt_tampon2, "done");
	}
    }
  hopla_message (txt_tampon1, "end");
  return TRUE;
}

/*****************************************************************************/
/* Local main */
/*****************************************************************************/
gboolean xml2struc_main (void)
{
  Td_xmlbase new_base;
  gchar *txt_tampon = NULL;

  /* Lecture du fichier */
  txt_tampon = g_strdup_printf (_("Reading file '%s'"), HOPLA_FILE_POPT);
  hopla_message (txt_tampon, "begin");
  new_base = td_parse_xmlbase (HOPLA_FILE_POPT);
  if (!new_base)
    {
      hopla_message (txt_tampon, "failed");
      hopla_message (txt_tampon, "end");
      printf (_("Error : File unreadable"));
      return FALSE;
    }
  hopla_message (txt_tampon, "done");
  hopla_message (txt_tampon, "end");

  /* Vrification du fichier */
  txt_tampon = g_strdup_printf (_("Checking file '%s' : database's structure"), HOPLA_FILE_POPT);
  hopla_message (txt_tampon, "begin");
  if (!check_name (new_base))
    {
      hopla_message (txt_tampon, "failed");
      hopla_message (txt_tampon, "end");
      return FALSE;
    }
  if (!check_type (new_base))
    {
      hopla_message (txt_tampon, "failed");
      hopla_message (txt_tampon, "end");
      return FALSE;
    }
  hopla_message (txt_tampon, "done");
  hopla_message (txt_tampon, "end");

  /* Ecriture base donnes : table */
  txt_tampon = g_strdup_printf (_("Writting database's structure '%s'"), HOPLA_BASE_POPT);
  hopla_message (txt_tampon, "begin");
  if (!hopla_write_dbbase (new_base))
    {
      hopla_message (txt_tampon, "failed");
      hopla_message (txt_tampon, "end");
      return FALSE;
    }
  hopla_message (txt_tampon, "done");
  hopla_message (txt_tampon, "end");
  return TRUE;
}
