/*****************************************************************************/
/* data.c : Commandes relatives aux donnes de la base donnes 
 * data.c : Database's data commands
 *
 *
 * Greffon hOpla : Interface PostgreSQL
 * hOpla's plug-in : PostgreSQL interface
 * Copyright (c) 2000 Philippe Roy
 * Auteur - Author : Philippe Roy <ph_roy@yahoo.com>
 *
 *
 * Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier
 * sous les termes de la licence publique gnrale GNU telle qu'elle est publie par
 * la Free Software Foundation ; soit la version 2 de la licence, ou
 * (comme vous voulez) toute version ultrieure.
 *
 * Ce programme est distribu dans l'espoir qu'il sera utile,
 * mais SANS AUCUNE GARANTIE ; mme sans la garantie de
 * COMMERCIALIT ou d'ADQUATION A UN BUT PARTICULIER. Voir la
 * licence publique gnrale GNU pour plus de dtails.
 *
 * Vous devriez avoir reu une copie de la licence publique gnrale GNU
 * avec ce programme ; si ce n'est pas le cas, crivez  la Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/*****************************************************************************/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "main.h"
#include "structure.h"
#include "data.h"

/*****************************************************************************/
/* Adaptation syntaxique de la valeur pour une requte SQL juste */
/*****************************************************************************/
gchar *adaptation_value (gchar *valeur, gchar *type)
{
  /* Analyse base sur le type (taille maxi du type = 50)
     Rgles :
     - nombre : x
     - caractre et chanes de caractres : 'x'
     - boolean : 'x'
     - l'apostrophe : ''
     - FIXME: La suite va viendre !
  */

  gchar *ret;
  gchar *txt_tampon;
  gchar *old_value;
  gchar *new_value;
  gchar *part_value;
  char char_tampon1[50];
  char char_tampon2[50];
  int i;

  /* boolean */
  if (!strcmp (type, "bool"))
    {
      /* Dlimiteur */
      valeur = g_strdup_printf ("'%s'", valeur);
      return (valeur);
    }

  /* char(n) */
  strcpy(char_tampon1, type);
  for (i=0; i<5; i++)
    char_tampon2[i] = char_tampon1[i];
  char_tampon2[i] = '\0';
  txt_tampon = g_strdup_printf ("%s", char_tampon2);
  if (!strcmp (txt_tampon, "char("))
    {
      valeur = hopla_strescape (valeur);
      valeur = g_strdup_printf ("'%s'", valeur);
      return (valeur);
    }

  /* varchar(n) */
  for (i=0; i<8; i++)
    char_tampon2[i] = char_tampon1[i];
  char_tampon2[i] = '\0';
  txt_tampon = g_strdup_printf ("%s", char_tampon2);
  if (!strcmp (txt_tampon, "varchar("))
    {
      valeur = hopla_strescape (valeur);
      valeur = g_strdup_printf ("'%s'", valeur);
      return (valeur);
    }
  return (valeur);
}


/*****************************************************************************/
/* Opration basiques */
/*****************************************************************************/
/* Addition des items */
gboolean add_data (PGconn *connexion, Td_xmlbase new_base)
{
  Td_xmltable new_table = NULL;
  Td_xmlfield new_field;
  Td_xmldatatable new_datatable;
  gchar *txt_tampon;
  gchar *requete;
  gchar *requete_fixe;
  int i, j, k;
  gboolean erreur;

  /* Go ! */
  for (k=0; k < new_base->nbdatatables; k++)
    {

      /* Composition de la requete : Table */
      new_datatable = new_base->datatables[k];
      requete_fixe = g_strdup_printf ("INSERT INTO %s (", new_datatable->table_source);

      /* Recherche de la table source dans le fichier */
      txt_tampon = g_strdup_printf (_("Searching file's table source '%s' in file's structure"), new_datatable->table_source);
      i = hopla_findindex_table (new_base, new_datatable->table_source);
      if (i == -1)
	{
	  hopla_message (txt_tampon, "failed");
	  return FALSE;
	}
      new_table = new_base->tables[i];
      hopla_message (txt_tampon, "done");

      /* Composition de la requete : Champs */
      for (i=0; i < new_datatable->nbfields; i++)
	{
	  requete_fixe = g_strdup_printf("%s%s", requete_fixe, new_table->fields[i]->name);
	  if (i != new_datatable->nbfields-1)
	    requete_fixe = g_strdup_printf("%s,", requete_fixe);
	  else
	    requete_fixe = g_strdup_printf("%s) ", requete_fixe);
	}

      /* Composition de la requete : Donnes */
      for (i=0; i < new_datatable->nbitems; i++)
	{
	  requete = g_strdup_printf ("VALUES (");
	  for (j=0; j < new_datatable->nbfields; j++)
	    {
	      new_field = new_table->fields[j];
	      requete = g_strdup_printf ("%s%s", requete, adaptation_value (new_datatable->items[i][j], new_field->type));
	      if (j != new_datatable->nbfields-1)
		requete = g_strdup_printf ("%s,", requete);
	      else
		requete = g_strdup_printf ("%s)", requete);
	    }
	  requete = g_strdup_printf ("%s%s", requete_fixe, requete);
	  if (!query_command (connexion, requete))
	    return FALSE;
	}
    }
  return TRUE;
}             	

/* Suppression des items */
gboolean remove_data (PGconn *connexion, Td_xmlbase new_base)
{
  Td_xmltable new_table = NULL;
  Td_xmlfield new_field;
  Td_xmldatatable new_datatable;
  gchar *txt_tampon;
  gchar *requete;
  gchar *requete_fixe;
  int i, j, k;
  gboolean erreur;

  /* Go ! */
  for (k=0; k < new_base->nbdatatables; k++)
    {

      /* Composition de la requete : Table */
      new_datatable = new_base->datatables[k];
      requete_fixe = g_strdup_printf ("DELETE FROM %s WHERE", new_datatable->table_source);

      /* Recherche de la table source dans le fichier */
      txt_tampon = g_strdup_printf (_("Searching file's table source '%s' in file's structure"), new_datatable->table_source);
      i = hopla_findindex_table (new_base, new_datatable->table_source);
      if (i == -1)
	{
	  hopla_message (txt_tampon, "failed");
	  return FALSE;
	}
      new_table = new_base->tables[i];
      hopla_message (txt_tampon, "done");

      /* Donnes */
      for (i=0; i < new_datatable->nbitems; i++)
	{
	  requete = g_strdup_printf ("%s", requete_fixe);
	  for (j=0; j < new_datatable->nbfields; j++)
	    {
	      new_field = new_table->fields[j];
	      requete = g_strdup_printf ("%s (%s=%s)", requete, new_field->name, adaptation_value (new_datatable->items[i][j], new_field->type));
	      if (j != new_datatable->nbfields-1)
		requete = g_strdup_printf ("%s AND", requete);
	      else
		requete = g_strdup_printf ("%s", requete);
	    }
	  requete = g_strdup_printf ("%s", requete);
	  if (!query_command (connexion, requete))
	    return FALSE;
	}
    }
  return TRUE;
}             	


/*****************************************************************************/
/* Mise  jour des donnes */
/*****************************************************************************/
gboolean update_data (PGconn *connexion, Td_xmlbase old_base, Td_xmlbase new_base)
{
  Td_xmltable old_table;
  Td_xmltable new_table;
  Td_xmlfield old_field;
  Td_xmlfield new_field;
  Td_xmldatatable old_datatable;
  Td_xmldatatable new_datatable;
  int new_index_hopla_id;
  int old_index_hopla_id;
  int i, j, k;
  gchar *requete;
  gchar *requete_fixe;
  gchar *txt_tampon;

  /* Rafrachissement de la base */
  for (i=0; i < new_base->nbdatatables; i++)
    {
      new_datatable = new_base->datatables[i];

      /* Recherche de la table source dans le fichier */
      txt_tampon = g_strdup_printf (_("Searching table '%s' in file's structure"), new_datatable->table_source);
      j = hopla_findindex_table (new_base, new_datatable->table_source);
      if (j == -1)
	{
	  hopla_message (txt_tampon, "failed");
	  return FALSE;
	}
      new_table = new_base->tables[j];
      hopla_message (txt_tampon, "done");

      /* Recherche de la table source dans la base de donnes */
      txt_tampon = g_strdup_printf (_("Searching table '%s' in database's structure"), new_datatable->table_source);
      j = hopla_findindex_table (old_base, new_datatable->table_source);
      if (j == -1)
	{
	  hopla_message (txt_tampon, "failed");
	  return FALSE;
	}
      old_table = old_base->tables[j];
      hopla_message (txt_tampon, "done");

      /* Recherche des champs administrateurs dans le fichier */
      txt_tampon = g_strdup_printf (_("Searching administation field '%s' file's structure"), "hopla_id");
      new_index_hopla_id = hopla_findindex_field (new_table, "hopla_id");
      if (new_index_hopla_id == -1)
	{
	  hopla_message (txt_tampon, "failed");
	  return FALSE;
	}
      txt_tampon = g_strdup_printf ("%s : field #%d", txt_tampon, new_index_hopla_id);
      hopla_message (txt_tampon, "done");

      /* Recherche des champs administrateurs dans la base de donnes */
      txt_tampon = g_strdup_printf (_("Searching administation field '%s' in database's structure"), "hopla_id");
      old_index_hopla_id = hopla_findindex_field (old_table, "hopla_id");
      if (old_index_hopla_id == -1)
	{
	  hopla_message (txt_tampon, "failed");
	  return FALSE;
	}
      txt_tampon = g_strdup_printf ("%s : field #%d", txt_tampon, old_index_hopla_id);
      hopla_message (txt_tampon, "done");

      /* Ajout des nouveaux : Table */
      requete_fixe = g_strdup_printf ("INSERT INTO %s (", new_datatable->table_source);

      /* Ajout des nouveaux : Champs */
      for (j=0; j < new_datatable->nbfields; j++)
	{
	  requete_fixe = g_strdup_printf("%s%s", requete_fixe, new_table->fields[j]->name);
	  if (j != new_datatable->nbfields-1)
	    requete_fixe = g_strdup_printf("%s,", requete_fixe);
	  else
	    requete_fixe = g_strdup_printf("%s) ", requete_fixe);
	}

      /* Ajout des nouveaux : Donnes */
      for (j=0; j < new_datatable->nbitems; j++)
	if (HOPLA_NEW_NODE[i][j])
	  {
	    requete = g_strdup_printf ("VALUES (");
	    for (k=0; k < new_datatable->nbfields; k++)
	      {
		new_field = new_table->fields[k];
		requete = g_strdup_printf ("%s%s", requete, adaptation_value (new_datatable->items[j][k], new_field->type));
		if (k != new_datatable->nbfields-1)
		  requete = g_strdup_printf ("%s,", requete);
		else
		  requete = g_strdup_printf ("%s);", requete);
	      }
	    requete = g_strdup_printf ("%s%s", requete_fixe, requete);
	    if (!query_command (connexion, requete))
	      return FALSE;
	  }

      /* Modification de l'existant : Table */
      requete_fixe = g_strdup_printf ("UPDATE %s", new_datatable->table_source);

      /* Modification de l'existant : Champs + Donnes */
      for (j=0; j < new_datatable->nbitems; j++)
	if (HOPLA_CHANGE_NODE[i][j])
	  {
	    requete = g_strdup_printf ("SET");
	    for (k=0; k < new_datatable->nbfields; k++)
	      if (k != new_index_hopla_id)
		{
		  new_field = new_table->fields[k];
		  requete = g_strdup_printf ("%s %s=%s", requete, new_field->name, adaptation_value (new_datatable->items[j][k], new_field->type));
		  if (k != new_datatable->nbfields-1)
		    requete = g_strdup_printf ("%s,", requete);
		  else
		    requete = g_strdup_printf ("%s", requete);
		}
	    requete = g_strdup_printf ("%s %s FROM %s WHERE hopla_id=%s", requete_fixe, requete, new_datatable->table_source, new_datatable->items[j][new_index_hopla_id]);
	    if (!query_command (connexion, requete))
	      return FALSE;
	  }

      /* Lecture de la table de donnes */
      requete = g_strdup_printf ("SELECT * FROM %s", new_datatable->table_source);
      txt_tampon = g_strdup_printf (_("Reading datatable '%s' in database"), new_datatable->table_source);
      hopla_message (txt_tampon, "begin");
      old_datatable = query_data (connexion, requete);
      if (!old_datatable)
	{
	  hopla_message (txt_tampon, "failed");
	  hopla_message (txt_tampon, "end");
	  return FALSE;
	}
      hopla_message (txt_tampon, "done");
      hopla_message (txt_tampon, "end");

      /* Mise en obsolescence */
      for (j=0; j < old_datatable->nbitems; j++)
	if (HOPLA_OBSO_NODE[i][j])
	  {
	    requete = g_strdup_printf ("%s SET hopla_obso='t' FROM %s WHERE hopla_id=%s", requete_fixe, new_datatable->table_source, old_datatable->items[j][old_index_hopla_id]);
	    if (!query_command (connexion, requete))
	      return FALSE;
	  }
    }
  return TRUE;
}


/*****************************************************************************/
/* Modification de la table de donnes */
/*****************************************************************************/
gboolean plugins_write_dbdatatable (Td_xmlbase new_base)
{
  Td_xmlbase old_base;
  PGconn *connexion;
  gchar *txt_tampon;

  /* Connexion */
  connexion = login ();
  if (!connexion)
    return FALSE;

  /* Lecture de la base de donnes */
  txt_tampon = g_strdup_printf (_("Reading database '%s'"), HOPLA_BASE_POPT);
  hopla_message (txt_tampon, "begin");
  old_base = parse_dbbase (connexion);
  if (!old_base)
    {
      hopla_message (txt_tampon, "failed");
      hopla_message (txt_tampon, "end");
      return FALSE;
    }
  hopla_message (txt_tampon, "done");
  hopla_message (txt_tampon, "end");

  /* Go ! */
  if (!strcmp (HOPLA_REFRESH_METHOD_POPT, "add"))
    {
      txt_tampon = g_strdup_printf ("%s", _("Adding items"));
      hopla_message (txt_tampon, "begin");
      if (!add_data (connexion, new_base))
	{
	  hopla_message (txt_tampon, "failed");
	  hopla_message (txt_tampon, "end");
	  logout (connexion);
	  return FALSE;
	}
      hopla_message (txt_tampon, "done");
      hopla_message (txt_tampon, "end");
    }

  if (!strcmp (HOPLA_REFRESH_METHOD_POPT, "remove"))
    {
      txt_tampon = g_strdup_printf ("%s", _("Removing items"));
      hopla_message (txt_tampon, "begin");
      if (!remove_data (connexion, new_base))
	{
	  hopla_message (txt_tampon, "failed");
	  hopla_message (txt_tampon, "end");
	  logout (connexion);
	  return FALSE;
	}
      hopla_message (txt_tampon, "done");
      hopla_message (txt_tampon, "end");
    }

  if (!strcmp (HOPLA_REFRESH_METHOD_POPT, "update"))
    {
      txt_tampon = g_strdup_printf ("%s", _("Updating items"));
      hopla_message (txt_tampon, "begin");
      if (!update_data (connexion, old_base, new_base))
	{
	  hopla_message (txt_tampon, "failed");
	  hopla_message (txt_tampon, "end");
	  logout (connexion);
	  return FALSE;
	}
      hopla_message (txt_tampon, "done");
      hopla_message (txt_tampon, "end");
    }

  /* Deconnexion */
  logout (connexion);
  return TRUE;
}


/*****************************************************************************/
/* Requete */
/*****************************************************************************/
Td_xmldatatable query_data (PGconn *connexion, char *requete)
{
  Td_xmldatatable ret = NULL;
  PGresult *resultat;
  int i, j;
  gchar *txt_tampon;
  char *requete_bis = NULL;
  ret = g_malloc0 (sizeof (Td_xmldatatable_tmp));

  /* Requete */
  txt_tampon = g_strdup_printf ("%s '%s'", _("Executing query"), requete);
  requete_bis = g_strdup_printf ("%s ;", requete);
  resultat = PQexec (connexion, requete_bis);
  if ((!resultat) || (PQresultStatus (resultat) != PGRES_TUPLES_OK))
    {
      hopla_message (txt_tampon, "failed");
      return (NULL);
    }
  hopla_message (txt_tampon, "command");

  /* Table */
  ret->table_source = g_strdup_printf ("%s", HOPLA_TABLE_POPT);

  /* Items */
  for (i = 0; i < PQntuples (resultat); i++)
    for (j = 0; j < PQnfields (resultat); j++)
      ret->items[i][j] = g_strdup_printf ("%s", PQgetvalue (resultat, i, j));
  ret->nbfields = PQnfields (resultat);
  ret->nbitems = PQntuples (resultat);
  return (ret);
}

Td_xmldatatable plugins_query (char *requete)
{
  Td_xmldatatable ret;
  PGconn *connexion;

  /* Connexion */
  connexion = login ();
  if (!connexion)
    return (NULL);

  /* Requete */
  ret = query_data (connexion, requete);

  /* Deconnexion */
  logout (connexion);
  return (ret);
}
