/*****************************************************************************/
/* main.c : Donnes globales et fentre principale
 * main.c : Global data and main window
 *
 *
 * Greffon hOpla : Interface PostgreSQL
 * hOpla's plug-in : PostgreSQL interface
 * Copyright (c) 2000 Philippe Roy
 * Auteur - Author : Philippe Roy <ph_roy@yahoo.com>
 *
 *
 * Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier
 * sous les termes de la licence publique gnrale GNU telle qu'elle est publie par
 * la Free Software Foundation ; soit la version 2 de la licence, ou
 * (comme vous voulez) toute version ultrieure.
 *
 * Ce programme est distribu dans l'espoir qu'il sera utile,
 * mais SANS AUCUNE GARANTIE ; mme sans la garantie de
 * COMMERCIALIT ou d'ADQUATION A UN BUT PARTICULIER. Voir la
 * licence publique gnrale GNU pour plus de dtails.
 *
 * Vous devriez avoir reu une copie de la licence publique gnrale GNU
 * avec ce programme ; si ce n'est pas le cas, crivez  la Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/*****************************************************************************/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "main.h"
#include "structure.h"
#include "data.h"


/*****************************************************************************/
/* Variables globales */
/*****************************************************************************/

/*****************************************************************************/
/* Initialisation */
/*****************************************************************************/
/* Identification */
Td_about_plugins plugins_about (void)
{
  Td_about_plugins ret = NULL;
  ret = g_malloc0 (sizeof (Td_about_plugins_tmp));
  ret->name = "PostgreSQL interface";
  ret->type = _("database interface");
  ret->version = "1.0.1";
  ret->blurb = _("Login, logout & SQL commands");
  ret->authors = "Philippe ROY <ph_roy@yahoo.com>";
  ret->help = "";
  ret->path1 = _("database");
  ret->path2 = "interface";
  ret->path3 = "PostgreSQL";
  ret->copyright = "Copyright (c) 2000 Philippe ROY";
  ret->license = _("GNU General Public License");
  ret->web = "";
  return ret;
}


/*****************************************************************************/
/* Connexion */
/*****************************************************************************/
PGconn *login (void)
{
  PGconn *ret;
  gchar *txt_tampon;

  /* Connexion */
  txt_tampon = g_strdup_printf ("%s : %s '%s', %s '%s', %s '%s' and %s '%s'", _("Loging"), _("hote"), HOPLA_HOTE_POPT, _("port"), HOPLA_PORT_POPT, _("database"), HOPLA_BASE_POPT, _("user"), HOPLA_USER_POPT);
  ret = PQsetdbLogin (HOPLA_HOTE_POPT, HOPLA_PORT_POPT, NULL, NULL, HOPLA_BASE_POPT, HOPLA_USER_POPT, HOPLA_PASSWORD_POPT);
  if (PQstatus (ret) == CONNECTION_BAD)
    {
      hopla_message (txt_tampon, "failed");
      return (NULL);
    }
  hopla_message (txt_tampon, "done");
  return ret;
}

void logout (PGconn *connexion)
{
  /* Dconnexion */
  gchar *txt_tampon;
  txt_tampon = g_strdup_printf ("%s : %s '%s', %s '%s', %s '%s' and %s '%s'", _("Logout"), _("hote"), HOPLA_HOTE_POPT, _("port"), HOPLA_PORT_POPT, _("database"), HOPLA_BASE_POPT, _("user"), HOPLA_USER_POPT);
  PQfinish (connexion);
  hopla_message (txt_tampon, "done");
}


/*****************************************************************************/
/* Commande */
/*****************************************************************************/
gboolean query_command (PGconn *connexion, char *requete)
{
  PGresult *resultat;
  gchar *txt_tampon;
  char *requete_bis = NULL;

  /* Requete */
  txt_tampon = g_strdup_printf ("%s '%s'", _("Executing query"), requete);
  requete_bis = g_strdup_printf ("%s ;", requete);
  if (HOPLA_TEST_MODE_POPT)
    {
      hopla_message (txt_tampon, "command");
      return TRUE;
    }
  resultat = PQexec (connexion, requete_bis);
  if (PQresultStatus (resultat) != PGRES_COMMAND_OK)
    {
      hopla_message (txt_tampon, "failed");
      return FALSE;
    }
  hopla_message (txt_tampon, "done");
  return TRUE;
}


/*****************************************************************************/
/* Administration */
/*****************************************************************************/
/* Lecture des donnes administrateur du gestionnaire de base de donnes */
Td_xmldatatable plugins_admin_read (void)
{
  Td_xmldatatable ret = NULL;
  PGconn *connexion;
  gchar *base_name = "template1";
  gchar *txt_tampon;

  /* Connexion */
  txt_tampon = g_strdup_printf ("%s : %s '%s', %s '%s', %s '%s' and %s '%s'", _("Loging"), _("host"), HOPLA_HOTE_POPT, _("port"), HOPLA_PORT_POPT, _("database"), base_name, _("user"), HOPLA_USER_POPT);
  connexion = PQsetdbLogin (HOPLA_HOTE_POPT, HOPLA_PORT_POPT, NULL, NULL, base_name, HOPLA_USER_POPT, HOPLA_PASSWORD_POPT);
  if (PQstatus (connexion) == CONNECTION_BAD)
    {
      hopla_message (txt_tampon, "failed");
      return (NULL);
    }
  hopla_message (txt_tampon, "done");

  /* Requete */
  ret = query_data (connexion, "SELECT datname FROM pg_database");

  /* Deconnexion */
  logout (connexion);
  return (ret);
}


/* Ecriture des donnes administrateur du gestionnaire de base de donnes */
gboolean plugins_admin_write (Td_xmldatatable new_datatable, Td_xmldatatable old_datatable)
{
  int i, j, system_verif;
  gboolean bool_tampon;
  gchar *txt_tampon;

  /* Creation des nouvelles bases */
  bool_tampon = TRUE;
  for (i=0; i<new_datatable->nbitems; i++)
    {
      bool_tampon = TRUE;
      txt_tampon = g_strdup_printf (_("Writting administration : base '%s' is new ?"), new_datatable->items[i][0]);
      for (j=0; j<old_datatable->nbitems; j++)
	if (!strcmp (new_datatable->items[i][0], old_datatable->items[j][0]))
	  {
	    bool_tampon = FALSE;
	    hopla_message (txt_tampon, "no");
	    break;
	  }
      if (bool_tampon)
	{
	  hopla_message (txt_tampon, "yes");
	  txt_tampon = g_strdup_printf (_("Add base in server : command 'createdb %s'"), new_datatable->items[i][0]);
	  system_verif = system (g_strdup_printf("createdb %s", new_datatable->items[i][0]));
	  if (system_verif != 0)
	    {
	      hopla_message (txt_tampon, "failed");
	      return FALSE;
	    }
	  else
	    hopla_message (txt_tampon, "done");
	}
    }

  /* Supression des bases obsoltes */
  bool_tampon = TRUE;
  for (i=0; i<old_datatable->nbitems; i++)
    {
      bool_tampon = TRUE;
      txt_tampon = g_strdup_printf (_("Writting administration : base '%s' is obsoleted ?"), old_datatable->items[i][0]);
      for (j=0; j<new_datatable->nbitems; j++)
	if (!strcmp (new_datatable->items[j][0], old_datatable->items[i][0]))
	  {
	    bool_tampon = FALSE;
	    hopla_message (txt_tampon, "no");
	    break;
	  }
      if (bool_tampon)
	{
	  hopla_message (txt_tampon, "yes");
	  txt_tampon = g_strdup_printf (_("Remove base in server : command 'dropdb %s'"), old_datatable->items[i][0]);
	  system_verif = system (g_strdup_printf("dropdb %s", old_datatable->items[i][0]));
	  if (system_verif != 0)
	    {
	      hopla_message (txt_tampon, "failed");
	      return FALSE;
	    }
	  else
	    hopla_message (txt_tampon, "done");
	}
    }
  return TRUE;
}


/*****************************************************************************/
/* Lecture de la base complte structure + donnes */
/*****************************************************************************/
Td_xmlbase plugins_read_dbbase (Td_xmlbase template)
{
  Td_xmlbase ret;
  Td_xmltable table;
  Td_xmlview view;
  Td_xmldatatable datatable;
  PGconn *connexion;
  int i, j;
  gchar *txt_tampon;
  gchar *requete;
  ret = g_malloc0 (sizeof (Td_xmlbase_tmp));
  table = g_malloc0 (sizeof (Td_xmltable_tmp));
  view = g_malloc0 (sizeof (Td_xmlview_tmp));
  datatable = g_malloc0 (sizeof (Td_xmldatatable_tmp));

  /* Connexion */
  connexion = login ();
  if (!connexion)
    {
      g_free (table);
      g_free (view);
      g_free (datatable);
      g_free (ret);
      return (NULL);
    }

  /* Structure et vues */
  if (HOPLA_TEMPLATE_POPT)
    ret = template;
  else
    {
      ret = parse_dbbase (connexion);
      ret->nbviews = 0;
    }

  /* Donnes avec garabit */
  if (HOPLA_TEMPLATE_POPT)
    {
      for (i=0; i < template->nbviews; i++)
	{

	  /* Champs */
	  view = template->views[i];
	  txt_tampon = g_strdup_printf (_("Reading view '%s'"), view->name);
	  hopla_message (txt_tampon, "begin");
	  requete = g_strdup_printf ("SELECT ");
	  for (j=0; j < view->nbfields; j++)
	    {
	      requete = g_strdup_printf ("%s%s", requete, view->fields[j]->name);
	      if (j != view->nbfields-1)
		requete = g_strdup_printf ("%s,", requete);
	      else
		requete = g_strdup_printf ("%s", requete);
	    }

	  /* Obsoltes */
	  if (HOPLA_OBSO_MODE_POPT)
	    requete = g_strdup_printf ("%s FROM %s ORDER BY hopla_id", requete, view->name);
	  else
	    requete = g_strdup_printf ("%s FROM %s WHERE hopla_obso = 'f' ORDER BY hopla_id", requete, view->name);

	  /* Go ! */
	  datatable = query_data (connexion, requete);
	  if (!datatable)
	    {
	      hopla_message (txt_tampon, "failed");
	      hopla_message (txt_tampon, "end");
	      g_free (table);
	      g_free (view);
	      g_free (datatable);
	      g_free (ret);
	      return (NULL);
	    }
	  datatable->table_source = g_strdup_printf ("%s", view->name);
	  hopla_message (txt_tampon, "done");
	  hopla_message (txt_tampon, "end");
	  ret->datatables[i] = datatable;
	}
      ret->nbdatatables = template->nbviews;
    }

  /* Donnes sans garabit */
  else
    {
      for (i=0; i < ret->nbtables; i++)
	{

	  /* Champs */
	  table = ret->tables[i];
	  txt_tampon = g_strdup_printf (_("Reading table '%s'"), table->name);
	  hopla_message (txt_tampon, "begin");
	  requete = g_strdup_printf ("SELECT ");
	  for (j=0; j < table->nbfields; j++)
	    {
	      requete = g_strdup_printf ("%s%s", requete, table->fields[j]->name);
	      if (j != table->nbfields-1)
		requete = g_strdup_printf ("%s,", requete);
	      else
		requete = g_strdup_printf ("%s", requete);
	    }

	  /* Obsoltes */
	  if (HOPLA_OBSO_MODE_POPT)
	    requete = g_strdup_printf ("%s FROM %s ORDER BY hopla_id", requete, table->name);
	  else
	    requete = g_strdup_printf ("%s FROM %s WHERE hopla_obso = 'f' ORDER BY hopla_id", requete, table->name);

	  /* Go ! */
	  datatable = query_data (connexion, requete);
	  if (!datatable)
	    {
	      hopla_message (txt_tampon, "failed");
	      hopla_message (txt_tampon, "end");
	      g_free (table);
	      g_free (view);
	      g_free (datatable);
	      g_free (ret);
	      return (NULL);
	    }
	  datatable->table_source = g_strdup_printf ("%s", table->name);
	  hopla_message (txt_tampon, "done");
	  hopla_message (txt_tampon, "end");
	  ret->datatables[i] = datatable;
	}
      ret->nbdatatables = ret->nbtables;
    }

  /* Deconnexion */
  logout (connexion);
/*    g_free (table); */
/*    g_free (view); */
/*    g_free (datatable); */
  return ret;
}
