/*****************************************************************************/
/* structure.c : Commandes relatives  la structure de la base donnes 
 * structure.c : Database's structure commands
 *
 *
 * Greffon hOpla : Interface PostgreSQL
 * hOpla's plug-in : PostgreSQL interface
 * Copyright (c) 2000 Philippe Roy
 * Auteur - Author : Philippe Roy <ph_roy@yahoo.com>
 *
 *
 * Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier
 * sous les termes de la licence publique gnrale GNU telle qu'elle est publie par
 * la Free Software Foundation ; soit la version 2 de la licence, ou
 * (comme vous voulez) toute version ultrieure.
 *
 * Ce programme est distribu dans l'espoir qu'il sera utile,
 * mais SANS AUCUNE GARANTIE ; mme sans la garantie de
 * COMMERCIALIT ou d'ADQUATION A UN BUT PARTICULIER. Voir la
 * licence publique gnrale GNU pour plus de dtails.
 *
 * Vous devriez avoir reu une copie de la licence publique gnrale GNU
 * avec ce programme ; si ce n'est pas le cas, crivez  la Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA. */
/*****************************************************************************/

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>

#include "main.h"
#include "structure.h"


/*****************************************************************************/
/* Analyse base de donnes */
/*****************************************************************************/
Td_xmlfield parse_dbfield (PGconn *connexion, int field_reference, gchar *field_name, int type_reference, int type_bytes)
{
  Td_xmlfield ret = NULL;
  PGresult *resultat;
  char *requete;
  int i;
  ret = g_malloc0 (sizeof (Td_xmlfield_tmp));

  /* Champs et type
     requete = g_strdup_printf("SELECT typname
     FROM pg_type
     WHERE oid = '%d'
     ORDER BY typname;",cur_champs->type_reference); */
  requete = g_strdup_printf ("SELECT typname FROM pg_type WHERE oid = '%d' ORDER BY typname;", type_reference);
  resultat = PQexec (connexion, requete);
  if ((!resultat) || (PQresultStatus (resultat) != PGRES_TUPLES_OK))
    return (NULL);

  /* Champs */
  ret->name = g_strdup_printf ("%s", field_name);
  ret->comment = g_strdup_printf ("(bud)");

  /* Type */
  ret->type = g_strdup_printf ("%s", PQgetvalue (resultat, 0, 0));
  ret->type_comment = g_strdup_printf ("(bud)");
  if (!strcmp (ret->type,"bpchar"))
    ret->type = g_strdup_printf ("char(%d)", (type_bytes)-4);
  if (!strcmp (ret->type,"varchar"))
    ret->type = g_strdup_printf ("varchar(%d)", (type_bytes)-4);

  /* Cl
     requete = g_strdup_printf("SELECT a.attname
     FROM pg_index i, pg_attribute a
     WHERE i.indrelid = '%d' AND
     i.indexrelid = a.attrelid AND
     a.attname = '%s' ;",
     cur_champs->reference, cur_champs->nom); */
  requete = g_strdup_printf ("SELECT a.attname FROM pg_index i, pg_attribute a WHERE i.indrelid = '%d' AND i.indexrelid = a.attrelid AND a.attname = '%s';", field_reference, field_name);
  resultat = PQexec (connexion, requete);
  if ((!resultat) || (PQresultStatus (resultat) != PGRES_TUPLES_OK))
    return (NULL);
	
  if (PQntuples (resultat) == 1)
    ret->key = 1;
  else
    if (PQntuples (resultat) == 0)
      ret->key = 0;
    else
      return (NULL);
  return (ret);
}

Td_xmltable parse_dbtable (PGconn *connexion, int table_reference, gchar *table_name)
{
  Td_xmltable ret = NULL;
  PGresult *resultat;
  char *requete;
  int i;
  int field_reference;
  gchar *field_name;
  int type_reference;
  int type_bytes;
  ret = g_malloc0 (sizeof (Td_xmltable_tmp));

  /* Requete
     requete = g_strdup_printf("SELECT attrelid, attname, atttypid, atttypmod
     FROM pg_attribute
     WHERE attrelid = '%d' AND
     attnum >0
     ORDER BY oid;",cur_table->reference); */
  requete = g_strdup_printf ("SELECT attrelid, attname, atttypid, atttypmod FROM pg_attribute WHERE attrelid = '%d' AND attnum >0 ORDER BY oid;", table_reference);
  resultat = PQexec (connexion, requete);
  if ((!resultat) || (PQresultStatus (resultat) != PGRES_TUPLES_OK))
    return (NULL);

  /* Table */
  ret->name = g_strdup_printf ("%s", table_name);
  ret->comment = g_strdup_printf ("(bud)");
  ret->nbfields = PQntuples (resultat);

  /* Champs */
  for (i = 0; i < PQntuples (resultat); i++)
    {
      field_reference = atoi (PQgetvalue (resultat, i, 0));
      field_name = g_strdup_printf ("%s", PQgetvalue (resultat, i, 1));
      type_reference = atoi (PQgetvalue (resultat, i, 2));
      type_bytes = atoi (PQgetvalue (resultat, i, 3));
      ret->fields[i] = parse_dbfield (connexion, field_reference, field_name, type_reference, type_bytes);
    }
  return (ret);
}

Td_xmlbase parse_dbbase (PGconn *connexion)
{
  Td_xmlbase ret = NULL;
  PGresult *resultat;
  char *requete;
  gchar *table_name;
  int i;
  int table_reference;
  ret = g_malloc0 (sizeof (Td_xmlbase_tmp));

  /* Requete
     requete = g_strdup_printf(" SELECT oid, relname
     FROM pg_class
     WHERE relkind ='r' AND
     relname !~ 'pg_'
     ORDER BY oid;"); */
  requete = g_strdup_printf ("SELECT oid, relname FROM pg_class WHERE relkind ='r' AND relname !~ 'pg_' ORDER BY oid;");
  resultat = PQexec (connexion, requete);
  if ((!resultat) || (PQresultStatus (resultat) != PGRES_TUPLES_OK))
    return (NULL);

  /* Base */
  ret->name = g_strdup_printf ("%s", HOPLA_BASE_POPT);
  ret->comment = g_strdup_printf ("(bud)");
  ret->nbtables = PQntuples (resultat);

  /* Tables */
  for (i = 0; i < PQntuples (resultat); i++)
    {
      table_reference = atoi (PQgetvalue (resultat, i, 0));
      table_name = g_strdup_printf("%s", PQgetvalue (resultat, i, 1));
      ret->tables[i] = parse_dbtable (connexion, table_reference, table_name);
    }
  return (ret);
}

Td_xmlbase plugins_parse_dbbase (void)
{
  Td_xmlbase ret;
  PGconn *connexion;

  /* Connexion */
  connexion = login ();
  if (!connexion)
    return (NULL);

  /* Analyse */
  ret = parse_dbbase (connexion);

  /* Deconnexion */
  logout (connexion);
  return (ret);
}


/*****************************************************************************/
/* Cration et destruction des tables */
/*****************************************************************************/
/* Cration d'une table */
gboolean table_create (PGconn *connexion, Td_xmltable table)
{
  Td_xmlfield field;
  PGresult *resultat;
  char *requete;
  int i;

  /* Go ! */
  requete = g_strdup_printf ("CREATE TABLE %s (", table->name);
  for (i=0; i < table->nbfields; i++)
    {
      field = table->fields[i];	
      requete = g_strdup_printf ("%s%s %s", requete, field->name, field->type);
      if (field->key)
	requete = g_strdup_printf ("%s PRIMARY KEY", requete);
      if (i != table->nbfields-1)
	requete = g_strdup_printf ("%s,", requete);
    }
  requete = g_strdup_printf ("%s);", requete);
  if (!query_command (connexion, requete))
    return FALSE;
  return TRUE;
}

/* Suppression d'une table */
gboolean table_remove (PGconn *connexion, Td_xmltable table)
{
  PGresult *resultat;
  gchar *requete;

  /* Go ! */
  requete = g_strdup_printf ("DROP TABLE %s;", table->name);
  if (!query_command (connexion, requete))
    return FALSE;
  return TRUE;
}

/* Renomer une table */
gboolean table_rename (PGconn *connexion, Td_xmltable table)
{
  /* Petit dtail : changement de nom de table ou champs = suppression + cration => perte des donnes */
  /* Little detail : table or field's name changed = remove + create => data in garbage, it's really trash */
  /* FIXME: +tards */
  return TRUE;
}

/* Ecriture des tables */
gboolean table_write (PGconn *connexion, Td_xmlbase old_base, Td_xmlbase new_base)
{
  Td_xmltable old_table = NULL;
  Td_xmltable new_table = NULL;
  int i, j;
  gboolean creation = TRUE;
  gboolean suppression = TRUE;
  gchar *txt_tampon;

  /* Table  crer */
  /* Scan table nouvelle */
  for (i=0; i < new_base->nbtables; i++)
    {
      new_table = new_base->tables[i];
      txt_tampon = g_strdup_printf ("%s : '%s' : %s ?", _("Creating table"), new_table->name, _("new"));
      creation = TRUE;

      /* Scan table actuelle */
      for (j=0; j < old_base->nbtables; j++)
	{
	  old_table = old_base->tables[j];
	  if (!strcmp (old_table->name, new_table->name))
	    creation = FALSE;
	}

      /* Cration table */
      if (creation)
	{
	  hopla_message (txt_tampon, "yes");
	  if (!table_create (connexion, new_table))
	    return FALSE;
	}
      else
	hopla_message (txt_tampon, "no");
    }

  /* Table  virer */
  /* Scan table actuelle */
  for (i=0; i < old_base->nbtables; i++)
    {
      old_table = old_base->tables[i];
      txt_tampon = g_strdup_printf ("%s : '%s' : %s ?", _("Removing table"), old_table->name, _("obsoleted"));
      suppression = TRUE;

      /* Scan table nouvelle */
      for (j=0; j < new_base->nbtables; j++)
	{
	  new_table = new_base->tables[j];
	  if (!strcmp (old_table->name, new_table->name))
	    suppression = FALSE;
	}

      /* Suppression table */
      if (suppression)
	{
	  hopla_message (txt_tampon, "yes");
	  if (!table_remove (connexion, old_table))
	    return FALSE;
	}
      else
	hopla_message (txt_tampon, "no");
    }
  return TRUE;
}


/*****************************************************************************/
/* Cration et destruction des champs */
/*****************************************************************************/
/* Cration d'un champs */
gboolean field_create (PGconn *connexion, Td_xmltable table, Td_xmlfield field)
{
  char *requete;

  /* Go ! */
  requete = g_strdup_printf ("ALTER TABLE %s ADD COLUMN %s %s;", table->name, field->name, field->type);
  if (!query_command (connexion, requete))
    return FALSE;
  return TRUE;
}

/* Suppression d'un champs */
gboolean field_remove (PGconn *connexion, Td_xmltable table, Td_xmlfield field, Td_xmlbase new_base)
{
  /* FIXME: Pour le momment le "ALTER TABLE DROP COLUMN" n'est pas encore implment dans PostgreSQL -> regnration complte de la table , youpii !! */
  /* requete = g_strdup_printf("ALTER TABLE %s DROP COLUMN %s;", table->nom, champs->nom); */
  /* Renomer table en table temporaire, crer nouvelle table, copier table temporaire vers la nouvelle table puis supprimer table temporaire */
  Td_xmltable new_table = NULL;
  Td_xmltable tmp_table = NULL;
  Td_xmlfield new_field = NULL;
  PGresult *resultat;
  gchar *requete;
  int i;

  /* Renomer table en table temporaire */
  requete = g_strdup_printf ("ALTER TABLE %s RENAME TO tmp;", table->name);
  if (!query_command (connexion, requete))
    return FALSE;

  /* Rechercher la nouvelle table  partir du nom de l'ancienne */
  for (i=0; i < new_base->nbtables; i++)
    {
      tmp_table = new_base->tables[i];	
      if (!strcmp (tmp_table->name, table->name))
	new_table = new_base->tables[i];
    }
  if (!new_table)
    {
      hopla_message (_("Searching new table for remove a old field in old table"), "failed");
      return FALSE;
    }

  /* Crer une nouvelle table */
  requete = g_strdup_printf ("CREATE TABLE %s (", new_table->name);
  for (i=0; i < new_table->nbfields; i++)
    {
      new_field = new_table->fields[i];	
      requete = g_strdup_printf ("%s%s %s", requete, new_field->name, new_field->type);
      if (i != new_table->nbfields-1)
	requete = g_strdup_printf ("%s,", requete);
      else
	requete = g_strdup_printf("%s)", requete);
    }
  requete = g_strdup_printf ("%s;", requete);
  if (!query_command (connexion, requete))
    return FALSE;

  /* Copier donnes *//* FIXME: + tard */

  /* Supprimer table temporaire */
  requete = g_strdup_printf ("DROP TABLE tmp;");
  if (!query_command (connexion, requete))
    return FALSE;
  return TRUE;
}

/* Renomer un champs */
gboolean field_rename (PGconn *connexion, Td_xmltable table, Td_xmlfield field, Td_xmlbase new_base)
{
  /* FIXME: +tards */
  return TRUE;
}

/* Comparaison entre ancienne et nouvelle structure pour prise de dcision (a rigole pas) */
gboolean field_write (PGconn *connexion, Td_xmlbase old_base, Td_xmlbase new_base)
{
  Td_xmltable new_table = NULL;
  Td_xmlfield new_field = NULL;
  Td_xmltable old_table = NULL;
  Td_xmlfield old_field = NULL;
  gboolean creation = FALSE;
  gboolean suppression = TRUE;
  gchar *message = NULL;
  gchar *txt_tampon;
  int i, j, m, n;

  /* Champs  crer */
  /* Scan champs nouvelle */
  for (i=0; i < new_base->nbtables; i++)
    {
      new_table = new_base->tables[i];
      for (j=0; j < new_table->nbfields; j++)
	{
	  new_field = new_table->fields[j];
	  txt_tampon = g_strdup_printf ("%s : '%s.%s' : %s ?", _("Creating field"), new_table->name, new_field->name, _("new"));
	  creation = TRUE;

	  /* Scan champs actuels */
	  for (m=0; m < old_base->nbtables; m++)
	    {
	      old_table = old_base->tables[m];
	      for (n=0; n < old_table->nbfields; n++)
		{
		  old_field = old_table->fields[n];
		  if (!strcmp (old_table->name, new_table->name))	
		    if (!strcmp (old_field->name, new_field->name))
		      creation = FALSE;
		}
	    }

	  /* Cration champs */
	  if (creation)
	    {
	      hopla_message (txt_tampon, "yes");
	      if (!field_create (connexion, new_table, new_field))
		return FALSE;
	    }
	  else
	    hopla_message (txt_tampon, "no");
	}
    }

  /* Champs  virer */
  /* Scan champs actuels */
  for (i=0; i < new_base->nbtables; i++)
    {
      new_table = old_base->tables[i];
      for (j=0; j < old_table->nbfields; j++)
	{
	  old_field = old_table->fields[j];
	  txt_tampon = g_strdup_printf ("%s : '%s.%s' : %s ?", _("Removing field"), old_table->name, old_field->name, _("obsoleted"));
	  suppression = TRUE;

	  /* Scan champs nouvelle */
	  for (m=0; m < new_base->nbtables; m++)
	    {
	      new_table = new_base->tables[m];
	      for (n=0; n < new_table->nbfields; n++)
		{
		  new_field = new_table->fields[n];
		  if (!strcmp (old_table->name, new_table->name))
		    if (!strcmp (old_field->name, new_field->name))
		      suppression = FALSE;
		}
	    }

	  /* Suppression champs ? */
	  if (suppression)
	    {
	      hopla_message (txt_tampon, "yes");
	      if (!field_remove (connexion, old_table, old_field, new_base))
		return FALSE;
	      j = old_table->nbfields; /* Du fait qu'il y a regnration complte sur la nouvelle structure de la table -> 1 fois suffit */
	    }
	  else
	    hopla_message (txt_tampon, "no");
	}
    }
  return TRUE;
}


/*****************************************************************************/
/* Modification des types */
/*****************************************************************************/
/* Modification d'un type */
gboolean type_rename (PGconn *connexion, Td_xmltable table, Td_xmlfield field)
{
  /* Attention  la compatibilit entre les deux types - Be careful compatibility between old and new type */
  /* FIXME: Pour le momment le "ALTER TABLE DROP COLUMN" n'est pas encore implment dans PostgreSQL -> regnration complte de la table , youpii !! */
  /* requete = g_strdup_printf("ALTER TABLE %s DROP COLUMN %s;", table->nom, champs->nom); */
  /* Renomer table en table temporaire, crer nouvelle table, copier table temporaire vers la nouvelle table puis supprimer table temporaire */
  gchar *requete;
  PGresult *resultat;
  int i;

  /* Renomer table en table temporaire */
  requete = g_strdup_printf ("ALTER TABLE %s RENAME TO tmp;", table->name);
  if (!query_command (connexion, requete))
    return FALSE;

  /* Crer une nouvelle table */
  requete = g_strdup_printf ("CREATE TABLE %s (", table->name);
  for (i=0; i < table->nbfields; i++)
    {
      field = table->fields[i];
      requete = g_strdup_printf ("%s%s %s", requete, field->name, field->type);
      if (i != table->nbfields-1)
	requete = g_strdup_printf ("%s,", requete);
      else
	requete = g_strdup_printf ("%s)", requete);
    }
  requete = g_strdup_printf ("%s;", requete);
  if (!query_command (connexion, requete))
    return FALSE;

  /* Copier donnes *//* FIXME : + tard */

  /* Supprimer table temporaire */
  requete = g_strdup_printf("DROP TABLE tmp;");
  if (!query_command (connexion, requete))
    return FALSE;
  return TRUE;
}

/* Comparaison entre ancienne et nouvelle structure pour prise de dcision (a rigole pas) */
gboolean type_write (PGconn *connexion, Td_xmlbase old_base, Td_xmlbase new_base)
{
  Td_xmltable old_table = NULL;
  Td_xmltable new_table = NULL;
  Td_xmlfield old_field = NULL;
  Td_xmlfield new_field = NULL;
  int i, j, m, n;
  gboolean modif_type = TRUE;
  gchar *message = NULL;
  gchar *txt_tampon;

  /* Type  modifier */
  /* Scan champs nouvelle */
  for (i=0; i < new_base->nbtables; i++)
    {
      new_table = new_base->tables[i];
      for (j=0; j < new_table->nbfields; j++)
	{
	  new_field = new_table->fields[j];
	  txt_tampon = g_strdup_printf ("%s : '%s.%s.%s' : %s ?", _("Type replacing"), new_table->name, new_field->name, new_field->type, _("modified"));
	  modif_type = TRUE;
	  
	  /* Scan champs actuelle */
	  for (m=0; m < old_base->nbtables; m++)
	    {
	      old_table = old_base->tables[m];
	      for (n=0; n < old_table->nbfields; n++)
		{
		  old_field = old_table->fields[n];
		  if (!strcmp (old_table->name, new_table->name))	
		    if (!strcmp (old_field->name, new_field->name))
		      if (!strcmp (old_field->type, new_field->type))
			modif_type = FALSE;
		}
	    }
	  
	  /* Modification type */
	  if (modif_type)
	    {
	      hopla_message (txt_tampon, "yes");
	      if (!type_rename (connexion, new_table, new_field))
		return FALSE;
	      j = new_table->nbfields; /* Du fait qu'il y a regnration complte sur la nouvelle structure de la table -> 1 fois suffit */
	    }
	  else
	    hopla_message (txt_tampon, "no");
	}
    }
  return TRUE;
}
  

/*****************************************************************************/
/* Modification de la base */
/*****************************************************************************/
gboolean plugins_write_dbbase (Td_xmlbase new_base)
{
  PGconn *connexion;
  Td_xmlbase old_base;
  old_base = g_malloc0 (sizeof (Td_xmlbase_tmp));

  /* Connexion */
  connexion = login ();
  if (!connexion)
    return FALSE;

  /* Go ! */
  old_base = parse_dbbase (connexion);  
  if (!table_write (connexion, old_base, new_base))
    {
      logout (connexion);
      return FALSE;
    }
  if (HOPLA_TEST_MODE_POPT)
    {
      hopla_message (_("Test mode for writting the new field's structure isn't implemented."), NULL);
      hopla_message (_("Test mode for writting the new type's structure isn't implemented."), NULL);
      logout (connexion);
      return TRUE;
    }
  old_base = parse_dbbase (connexion);  
  if (!field_write (connexion, old_base, new_base))
    {
      logout (connexion);
      return FALSE;
    }
  old_base = parse_dbbase (connexion);  
  if (!type_write (connexion, old_base, new_base))
  {
    logout (connexion);
    return FALSE;
  }

  /* Deconnexion */
  logout (connexion);
  return TRUE;
}
